# Script for message generation
# Copyright (C) 2004,2005 Thomas Mathys (killer@vantage.ch)
#
# This file is part of rfk7800.
#
# rfk7800 is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# rfk7800 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rfk7800; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

SCREEN_WIDTH = 40

# Returns a copy of s with newlines added,
# so that the string fits onto the screen.
def addNewlines(s)
  s = s.clone
  textlen = s.length
  lastspace = 0
  lastwrap = 0
  textlen.times do |i|
    lastspace = i if s[i] == ?\s
    if (i-lastwrap) >= (SCREEN_WIDTH-1)
      s[lastspace] = ?\n
      lastwrap = lastspace
    end
  end
  return s
end

# Find out how many lines a message will need on the screen
def getNeededLinesOnScreen(s)
  ncolumns = 0
  nlines = 1
  s.each_byte do |char|
    ncolumns += 1
    if (char == ?\n) || (ncolumns > SCREEN_WIDTH)
      nlines += 1 
      ncolumns = 0
    end
  end
  return nlines
end

def writeMessageArray(messages, name)
  puts "#define #{name}_N (sizeof(#{name}) / sizeof(#{name}[0]))"
  longest = messages.max { |a,b| getNeededLinesOnScreen(a) <=> getNeededLinesOnScreen(b) }
  puts "#define #{name}_MAXLINES #{getNeededLinesOnScreen(longest)}"
  puts "const unsigned char * const #{name}[] = {"
  messages.each { |msg| puts "  #{addNewlines(msg).dump}," }
  puts "};"
end

# Extract string literals from lines.
# Assumes each line contains at most one C string literal.
# The resulting strings are unescaped, so "\n" becomes a single newline.
def getStringLiterals(lines)
  result = Array.new
  lines.each do |line|
    result.push(eval($&)) if line =~ /".*"/
  end
  return result
end

begin
  puts "#ifndef MESSAGES_H"
  puts "#define MESSAGES_H"
  puts
  msgs = getStringLiterals(File.open("messages.h") { |file| file.readlines })
  writeMessageArray(msgs, "NKI_MESSAGES")
  puts
  puts "#endif"
rescue StandardError => e
  $stderr.puts "Argh : #{e}"
  exit(1)
end

