/*
 * Atari 7800 register definitions.
 * Copyright (C) 2004,2005 Thomas Mathys (killer@vantage.ch)
 *
 * This file is part of rfk7800.
 *
 * rfk7800 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * rfk7800 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rfk7800; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef MARIA_H
#define MARIA_H


/*
 * TIA registers
 * (Only those that are relevant when in 7800 mode)
 */
struct __tia {
    unsigned char reserved0;
    unsigned char inptctrl;
    unsigned char reserved1[6];
    unsigned char inpt0;
    unsigned char inpt1;
    unsigned char inpt2;
    unsigned char inpt3;
    unsigned char inpt4;
    unsigned char inpt5;
    unsigned char reserved2[7];
    unsigned char audc0;
    unsigned char audc1;
    unsigned char audf0;
    unsigned char audf1;
    unsigned char audv0;
    unsigned char audv1;
};
#define TIA (*(struct __tia *)0x0000)


/*
 * Alternative TIA register definitions.
 */
#define INPTCTRL    TIA.inptctrl
#define PTCTRL      TIA.inptctrl
#define INPT0       TIA.inpt0
#define INPT1       TIA.inpt1
#define INPT2       TIA.inpt2
#define INPT3       TIA.inpt3
#define INPT4       TIA.inpt4
#define INPT5       TIA.inpt5
#define AUDC0       TIA.audc0
#define AUDC1       TIA.audc1
#define AUDF0       TIA.audf0
#define AUDF1       TIA.audf1
#define AUDV0       TIA.audv0
#define AUDV1       TIA.audv1


/*
 * MARIA registers
 */
struct __maria {
    unsigned char bkgrnd;
    unsigned char p0c1;
    unsigned char p0c2;
    unsigned char p0c3;
    unsigned char wsync;
    unsigned char p1c1;
    unsigned char p1c2;
    unsigned char p1c3;
    unsigned char mstat;
    unsigned char p2c1;
    unsigned char p2c2;
    unsigned char p2c3;
    unsigned char dpph;
    unsigned char p3c1;
    unsigned char p3c2;
    unsigned char p3c3;
    unsigned char dppl;
    unsigned char p4c1;
    unsigned char p4c2;
    unsigned char p4c3;
    unsigned char chbase;
    unsigned char p5c1;
    unsigned char p5c2;
    unsigned char p5c3;
    unsigned char offset;
    unsigned char p6c1;
    unsigned char p6c2;
    unsigned char p6c3;
    unsigned char ctrl;
    unsigned char p7c1;
    unsigned char p7c2;
    unsigned char p7c3;
};
#define MARIA   (*(struct __maria *)0x0020)


/*
 * Alternative MARIA register definitions.
 */
#define BKGRND      MARIA.bkgrnd
#define BACKGRND    MARIA.bkgrnd
#define P0C1        MARIA.p0c1
#define P0C2        MARIA.p0c2
#define P0C3        MARIA.p0c3
#define WSYNC       MARIA.wsync
#define P1C1        MARIA.p1c1
#define P1C2        MARIA.p1c2
#define P1C3        MARIA.p1c3
#define MSTAT       MARIA.mstat
#define P2C1        MARIA.p2c1
#define P2C2        MARIA.p2c2
#define P2C3        MARIA.p2c3
#define DPPH        MARIA.dpph
#define DPH         MARIA.dpph
#define P3C1        MARIA.p3c1
#define P3C2        MARIA.p3c2
#define P3C3        MARIA.p3c3
#define DPPL        MARIA.dppl
#define DPL         MARIA.dppl
#define P4C1        MARIA.p4c1
#define P4C2        MARIA.p4c2
#define P4C3        MARIA.p4c3
#define CHBASE      MARIA.chbase
#define P5C1        MARIA.p5c1
#define P5C2        MARIA.p5c2
#define P5C3        MARIA.p5c3
#define OFFSET      MARIA.offset
#define P6C1        MARIA.p6c1
#define P6C2        MARIA.p6c2
#define P6C3        MARIA.p6c3
#define CTRL        MARIA.ctrl
#define P7C1        MARIA.p7c1
#define P7C2        MARIA.p7c2
#define P7C3        MARIA.p7c3


/*
 * RIOT registers
 */
struct __riot {
    unsigned char swcha;
    unsigned char ctlswa;
    unsigned char swchb;
    unsigned char ctlswb;
    unsigned char intim;
    unsigned char reserved0[15];
    unsigned char tim1t;
    unsigned char tim8t;
    unsigned char tim64t;
    unsigned char t1024t;
};
#define RIOT    (*(struct __riot *)0x0280)


/*
 * Alternative RIOT register definitions.
 */
#define SWCHA       RIOT.swcha
#define CTLSWA      RIOT.ctlswa
#define SWCHB       RIOT.swchb
#define CTLSWB      RIOT.ctlswb
#define INTIM       RIOT.intim
#define TIM1T       RIOT.tim1t
#define TIM8T       RIOT.tim8t
#define TIM64T      RIOT.tim64t
#define T1024T      RIOT.t1024t


/*
 * Display list entry
 */
typedef struct {
    unsigned char offset;
    unsigned char addrhi;
    unsigned char addrlo;
}DLLEntry;


/*
 * A null header.
 * The unused field can be used to store data.
 */
typedef struct {
    unsigned char unused;
    unsigned char zero;
}NullHeader;


/*
 * Normal (4 byte) header
 */
typedef struct {
    unsigned char addrlo;
    unsigned char palwidth;
    unsigned char addrhi;
    unsigned char hpos;
}Header;


/*
 * Extended (5 byte) header
 */
typedef struct {
    unsigned char addrlo;
    unsigned char flags;
    unsigned char addrhi;
    unsigned char palwidth;
    unsigned char hpos;
}XHeader;


/*
 * MSTAT register constants
 */
#define MSTAT_VBLANK    0x80    /* vblank is on when bit 7 is set */


/*
 * CTRL register constants
 */
#define CTRL_CKOFF      0x00    /* color kill on/off */
#define CTRL_CKON       0x80
#define CTRL_DMAOFF     0x60    /* dma on/off */
#define CTRL_DMAON      0x40
#define CTRL_CHAR1B     0x00    /* one/two byte characters */
#define CTRL_CHAR2B     0x10
#define CTRL_BCBLACK    0x00    /* black border */
#define CTRL_BCBKGRND   0x08    /* border has background color */
#define CTRL_KANGOFF    0x00    /* kangaroo mode on/off */
#define CTRL_KANGON     0x04
#define CTRL_MODE160    0x00    /* mode 160x2 or 160x4 */
#define CTRL_MODEBD     0x02    /* mode 320b or 320d */
#define CTRL_MODEAC     0x03    /* mode 320a or 320c */


#endif
